#ifndef _MESH_NODE_H_
#define _MESH_NODE_H_

#include "cglib/reference.h"
#include "cglib/scene_graph_node.h"
#include "cglib/mesh.h"
#include "cglib/mesh_material_mapping.h"
#include "cglib/material_list.h"

//! Represent a node in the scene graph that contains a Mesh.
class MeshNode : public SceneGraphNode
{
public:
    //! A constructor that sets all the components to NULL.
    MeshNode ();
    
    //! A destructor that does nothing.
    virtual ~MeshNode ();
    
    //! Accept a visitor.
    virtual void accept(SceneGraphNodeVisitor *visitor);
    
    //! Return the mesh.
    Mesh *get_mesh();
    
    //! Return the material mapping of the mesh.
    MeshMaterialMapping *get_material_mapping();
    
    //! Return the material list associated with the mesh.
    MaterialList *get_material_list();
    
    //! Set the mesh of \a this to the given mesh.
    void set_mesh(Mesh *mesh);
    
    //! Set the material mapping of \a this to the given material mapping.
    void set_material_mapping(MeshMaterialMapping *material_mapping);
    
    //! Set the material list of \a this to the given material list.
    void set_material_list(MaterialList *material_list);

private:
    Reference<Mesh> mesh;                                   //!< the mesh
    Reference<MeshMaterialMapping> material_mapping;   //!< the mesh's material mapping
    Reference<MaterialList> material_list;                  //!< the mesh's material list
};

#endif /* _MESH_NODE_H_ */
