#ifndef _TRANSFORM_NODE_H_
#define _TRANSFORM_NODE_H_

#include "cglib/scene_graph_node.h"
#include "cglib/transform.h"
#include "cglib/reference.h"

//! Represent a transformation in the scene graph.
class TransformNode : public SceneGraphNode
{
public:
    //! A constructor that sets the transformation to the identity transformation, and set the child of the node to NULL.
    TransformNode ();
    
    //! A destructor that does nothing.
    virtual ~TransformNode ();
    
    //! Accept a visitor.
    virtual void accept(SceneGraphNodeVisitor *visitor);
    
    //! Get the transform from the child's space to the world space.
    Transform get_transform() const;
    
    //! Set the transform to the given transform.
    void set_transform(const Transform &_transform);
    
    //! Get the child.
    SceneGraphNode *get_child();
    
    //! Set the child.
    void set_child(SceneGraphNode *_child);
    
private:
    Reference<SceneGraphNode> child; //!< the child of the node
    Transform transform;             //!< the transfrom from the child's space to the world space.
};

#endif /* _TRANSFORM_NODE_H_ */
