#include "gtest/gtest.h"
#include "cglib/definitions.h"
#include "cglib/mesh.h"
#include "cglib/reference.h"

TEST(MeshTest, CubeTest)
{
	Reference<Mesh> cube(new Mesh());

	cube->append_position(Point3(-1,-1,-1));
	cube->append_position(Point3(-1,-1, 1));
	cube->append_position(Point3(-1, 1,-1));
	cube->append_position(Point3(-1, 1, 1));
	cube->append_position(Point3( 1,-1,-1));
	cube->append_position(Point3( 1,-1, 1));
	cube->append_position(Point3( 1, 1,-1));
	cube->append_position(Point3( 1, 1, 1));

	cube->append_normal(Vector3( 0, 0, 1));
	cube->append_normal(Vector3( 0, 0,-1));
	cube->append_normal(Vector3( 1, 0, 0));
	cube->append_normal(Vector3(-1, 0, 0));
	cube->append_normal(Vector3( 0, 1, 0));
	cube->append_normal(Vector3( 0,-1, 0));
	
	// The front face.
	cube->append_new_face();
	cube->append_vertex_to_last_face(1,0,-1);
	cube->append_vertex_to_last_face(5,0,-1);
	cube->append_vertex_to_last_face(7,0,-1);
	cube->append_vertex_to_last_face(3,0,-1);

	// The back face.
	cube->append_new_face();
	cube->append_vertex_to_last_face(2,1,-1);
	cube->append_vertex_to_last_face(6,1,-1);
	cube->append_vertex_to_last_face(4,1,-1);
	cube->append_vertex_to_last_face(0,1,-1);

	// The right face.
	cube->append_new_face();
	cube->append_vertex_to_last_face(5,2,-1);
	cube->append_vertex_to_last_face(4,2,-1);
	cube->append_vertex_to_last_face(6,2,-1);
	cube->append_vertex_to_last_face(7,2,-1);

	// The left face.
	cube->append_new_face();
	cube->append_vertex_to_last_face(3,3,-1);
	cube->append_vertex_to_last_face(2,3,-1);
	cube->append_vertex_to_last_face(0,3,-1);
	cube->append_vertex_to_last_face(1,3,-1);

	// The top face.
	cube->append_new_face();
	cube->append_vertex_to_last_face(3,4,-1);
	cube->append_vertex_to_last_face(7,4,-1);
	cube->append_vertex_to_last_face(6,4,-1);
	cube->append_vertex_to_last_face(2,4,-1);

	// The bottom face.
	cube->append_new_face();
	cube->append_vertex_to_last_face(0,5,-1);
	cube->append_vertex_to_last_face(4,5,-1);
	cube->append_vertex_to_last_face(5,5,-1);
	cube->append_vertex_to_last_face(1,5,-1);

	ASSERT_EQ(8, cube->get_position_count());
	ASSERT_EQ(6, cube->get_normal_count());
	ASSERT_EQ(6, cube->get_face_count());
	
	ASSERT_EQ(4, cube->get_face_vertex_count(0));
	ASSERT_EQ(4, cube->get_face_vertex_count(1));
	ASSERT_EQ(4, cube->get_face_vertex_count(2));
	ASSERT_EQ(4, cube->get_face_vertex_count(3));
	ASSERT_EQ(4, cube->get_face_vertex_count(4));
	ASSERT_EQ(4, cube->get_face_vertex_count(5));

	FOR(i,4) ASSERT_TRUE(Vector3( 0, 0, 1).is_near(cube->get_face_vertex_normal(0,i), EPSILON));
	FOR(i,4) ASSERT_TRUE(Vector3( 0, 0,-1).is_near(cube->get_face_vertex_normal(1,i), EPSILON));	
	FOR(i,4) ASSERT_TRUE(Vector3( 1, 0, 0).is_near(cube->get_face_vertex_normal(2,i), EPSILON));
	FOR(i,4) ASSERT_TRUE(Vector3(-1, 0, 0).is_near(cube->get_face_vertex_normal(3,i), EPSILON));
	FOR(i,4) ASSERT_TRUE(Vector3( 0, 1, 0).is_near(cube->get_face_vertex_normal(4,i), EPSILON));
	FOR(i,4) ASSERT_TRUE(Vector3( 0,-1, 0).is_near(cube->get_face_vertex_normal(5,i), EPSILON));

	ASSERT_TRUE(Point3(-1,-1,1).is_near(cube->get_face_vertex_position(0,0), EPSILON));
	ASSERT_TRUE(Point3( 1,-1,1).is_near(cube->get_face_vertex_position(0,1), EPSILON));
	ASSERT_TRUE(Point3( 1, 1,1).is_near(cube->get_face_vertex_position(0,2), EPSILON));
	ASSERT_TRUE(Point3(-1, 1,1).is_near(cube->get_face_vertex_position(0,3), EPSILON));
}